<?php
/**
 * @name: Kurdish Date and Time Functions
 * @Author : Lajan , https://lajan.ir
 * @License: GNU/LGPL _ Open Source & Free : [all functions]
 * @Version: 1.0 =>[ 2023/05/12 ]
**/

/*	F	*/
function kdate($format, $timestamp = '', $none = '', $time_zone = 'Asia/Tehran', $kr_num = 'فا') {

  $T_sec = 0;/* <= Remove server time errors, in numbers, '+' and '-' by seconds */

  if ($time_zone != 'local') date_default_timezone_set(($time_zone === '') ? 'Asia/Tehran' : $time_zone);
  $ts = $T_sec + (($timestamp === '') ? time() : kr_num($timestamp));
  $date = explode('_', date('H_i_j_n_O_P_s_w_Y', $ts));
  list($k_y, $k_m, $k_d) = gregorian_to_kurdi($date[8], $date[3], $date[2]);
  $doy = ($k_m < 7) ? (($k_m - 1) * 31) + $k_d - 1 : (($k_m - 7) * 30) + $k_d + 185;
  $kab = (((($k_y + 12) % 33) % 4) == 1) ? 1 : 0;
  $sl = strlen($format);
  $out = '';
  for ($i = 0; $i < $sl; $i++) {
    $sub = substr($format, $i, 1);
    if ($sub == '\\') {
      $out .= substr($format, ++$i, 1);
      continue;
    }
    switch ($sub) {

      case 'E':
      case 'R':
      case 'x':
      case 'X':
        $out .= 'https://lajan.ir';
        break;

      case 'B':
      case 'e':
      case 'g':
      case 'G':
      case 'h':
      case 'I':
      case 'T':
      case 'u':
      case 'Z':
        $out .= date($sub, $ts);
        break;

      case 'a':
        $out .= ($date[0] < 12) ? 'پێش.ن' : 'پاش.ن';
        break;

      case 'A':
        $out .= ($date[0] < 12) ? 'پێش نیوەڕۆ' : 'پاش نیوەڕۆ';
        break;

      case 'b':
        $out .= (int) ($k_m / 3.1) + 1;
        break;

      case 'c':
        $out .= $k_y . '/' . $k_m . '/' . $k_d . ' ،' . $date[0] . ':' . $date[1] . ':' . $date[6] . ' ' . $date[5];
        break;

      case 'C':
        $out .= (int) (($k_y + 99) / 100);
        break;

      case 'd':
        $out .= ($k_d < 10) ? '0' . $k_d : $k_d;
        break;

      case 'D':
        $out .= kdate_words(array('kh' => $date[7]), ' ');
        break;

      case 'f':
        $out .= kdate_words(array('ff' => $k_m), ' ');
        break;

      case 'F':
        $out .= kdate_words(array('mm' => $k_m), ' ');
        break;

      case 'H':
        $out .= $date[0];
        break;

      case 'i':
        $out .= $date[1];
        break;

      case 'j':
        $out .= $k_d;
        break;

      case 'J':
        $out .= kdate_words(array('rr' => $k_d), ' ');
        break;

      case 'k';
        $out .= kr_num(100 - (int) ($doy / ($kab + 365.24) * 1000) / 10, $kr_num);
        break;

      case 'K':
        $out .= kr_num((int) ($doy / ($kab + 365.24) * 1000) / 10, $kr_num);
        break;

      case 'l':
        $out .= kdate_words(array('rh' => $date[7]), ' ');
        break;

      case 'L':
        $out .= $kab;
        break;

      case 'm':
        $out .= ($k_m > 9) ? $k_m : '0' . $k_m;
        break;

      case 'M':
        $out .= kdate_words(array('km' => $k_m), ' ');
        break;

      case 'n':
        $out .= $k_m;
        break;

      case 'N':
        $out .= $date[7] + 1;
        break;

      case 'o':
        $jdw = ($date[7] == 6) ? 0 : $date[7] + 1;
        $dny = 364 + $kab - $doy;
        $out .= ($jdw > ($doy + 3) and $doy < 3) ? $k_y - 1 : (((3 - $dny) > $jdw and $dny < 3) ? $k_y + 1 : $k_y);
        break;

      case 'O':
        $out .= $date[4];
        break;

      case 'p':
        $out .= kdate_words(array('mb' => $k_m), ' ');
        break;

      case 'P':
        $out .= $date[5];
        break;

      case 'q':
        $out .= kdate_words(array('sh' => $k_y), ' ');
        break;

      case 'Q':
        $out .= $kab + 364 - $doy;
        break;

      case 'r':
        $key = kdate_words(array('rh' => $date[7], 'mm' => $k_m));
        $out .= $date[0] . ':' . $date[1] . ':' . $date[6] . ' ' . $date[4] . ' ' . $key['rh'] . '، ' . $k_d . ' ' . $key['mm'] . ' ' . $k_y;
        break;

      case 's':
        $out .= $date[6];
        break;

      case 'S':
        $out .= 'ەم';
        break;

      case 't':
        $out .= ($k_m != 12) ? (31 - (int) ($k_m / 6.5)) : ($kab + 29);
        break;

      case 'U':
        $out .= $ts;
        break;

      case 'v':
        $out .= kdate_words(array('ss' => ($k_y % 100)), ' ');
        break;

      case 'V':
        $out .= kdate_words(array('ss' => $k_y), ' ');
        break;

      case 'w':
        $out .= ($date[7] == 6) ? 0 : $date[7] + 1;
        break;

      case 'W':
        $avs = (($date[7] == 6) ? 0 : $date[7] + 1) - ($doy % 7);
        if ($avs < 0) $avs += 7;
        $num = (int) (($doy + $avs) / 7);
        if ($avs < 4) {
          $num++;
        } elseif ($num < 1) {
          $num = ($avs == 4 or $avs == ((((($k_y % 33) % 4) - 2) == ((int) (($k_y % 33) * 0.05))) ? 5 : 4)) ? 53 : 52;
        }
        $aks = $avs + $kab;
        if ($aks == 7) $aks = 0;
        $out .= (($kab + 363 - $doy) < $aks and $aks < 3) ? '01' : (($num < 10) ? '0' . $num : $num);
        break;

      case 'y':
        $out .= substr($k_y, 2, 2);
        break;

      case 'Y':
        $out .= $k_y;
        break;

      case 'z':
        $out .= $doy;
        break;

      default:
        $out .= $sub;
    }
  }
  return ($kr_num != 'en') ? kr_num($out, 'fa', '.') : $out;
}

/*	F	*/
function kstrftime($format, $timestamp = '', $none = '', $time_zone = 'Asia/Tehran', $kr_num = 'fa') {

  $T_sec = 0;/* <= Remove server time errors, in numbers, '+' and '-' by seconds */

  if ($time_zone != 'local') date_default_timezone_set(($time_zone === '') ? 'Asia/Tehran' : $time_zone);
  $ts = $T_sec + (($timestamp === '') ? time() : kr_num($timestamp));
  $date = explode('_', date('h_H_i_j_n_s_w_Y', $ts));
  list($k_y, $k_m, $k_d) = gregorian_to_kurdi($date[7], $date[4], $date[3]);
  $doy = ($k_m < 7) ? (($k_m - 1) * 31) + $k_d - 1 : (($k_m - 7) * 30) + $k_d + 185;
  $kab = (((($k_y + 12) % 33) % 4) == 1) ? 1 : 0;
  $sl = strlen($format);
  $out = '';
  for ($i = 0; $i < $sl; $i++) {
    $sub = substr($format, $i, 1);
    if ($sub == '%') {
      $sub = substr($format, ++$i, 1);
    } else {
      $out .= $sub;
      continue;
    }
    switch ($sub) {

        /* Day */
      case 'a':
        $out .= kdate_words(array('kh' => $date[6]), ' ');
        break;

      case 'A':
        $out .= kdate_words(array('rh' => $date[6]), ' ');
        break;

      case 'd':
        $out .= ($k_d < 10) ? '0' . $k_d : $k_d;
        break;

      case 'e':
        $out .= ($k_d < 10) ? ' ' . $k_d : $k_d;
        break;

      case 'j':
        $out .= str_pad($doy + 1, 3, 0, STR_PAD_LEFT);
        break;

      case 'u':
        $out .= $date[6] + 1;
        break;

      case 'w':
        $out .= ($date[6] == 6) ? 0 : $date[6] + 1;
        break;

        /* Week */
      case 'U':
        $avs = (($date[6] < 5) ? $date[6] + 2 : $date[6] - 5) - ($doy % 7);
        if ($avs < 0) $avs += 7;
        $num = (int) (($doy + $avs) / 7) + 1;
        if ($avs > 3 or $avs == 1) $num--;
        $out .= ($num < 10) ? '0' . $num : $num;
        break;

      case 'V':
        $avs = (($date[6] == 6) ? 0 : $date[6] + 1) - ($doy % 7);
        if ($avs < 0) $avs += 7;
        $num = (int) (($doy + $avs) / 7);
        if ($avs < 4) {
          $num++;
        } elseif ($num < 1) {
          $num = ($avs == 4 or $avs == ((((($k_y % 33) % 4) - 2) == ((int) (($k_y % 33) * 0.05))) ? 5 : 4)) ? 53 : 52;
        }
        $aks = $avs + $kab;
        if ($aks == 7) $aks = 0;
        $out .= (($kab + 363 - $doy) < $aks and $aks < 3) ? '01' : (($num < 10) ? '0' . $num : $num);
        break;

      case 'W':
        $avs = (($date[6] == 6) ? 0 : $date[6] + 1) - ($doy % 7);
        if ($avs < 0) $avs += 7;
        $num = (int) (($doy + $avs) / 7) + 1;
        if ($avs > 3) $num--;
        $out .= ($num < 10) ? '0' . $num : $num;
        break;

        /* Month */
      case 'b':
      case 'h':
        $out .= kdate_words(array('km' => $k_m), ' ');
        break;

      case 'B':
        $out .= kdate_words(array('mm' => $k_m), ' ');
        break;

      case 'm':
        $out .= ($k_m > 9) ? $k_m : '0' . $k_m;
        break;

        /* Year */
      case 'C':
        $tmp = (int) ($k_y / 100);
        $out .= ($tmp > 9) ? $tmp : '0' . $tmp;
        break;

      case 'g':
        $jdw = ($date[6] == 6) ? 0 : $date[6] + 1;
        $dny = 364 + $kab - $doy;
        $out .= substr(($jdw > ($doy + 3) and $doy < 3) ? $k_y - 1 : (((3 - $dny) > $jdw and $dny < 3) ? $k_y + 1 : $k_y), 2, 2);
        break;

      case 'G':
        $jdw = ($date[6] == 6) ? 0 : $date[6] + 1;
        $dny = 364 + $kab - $doy;
        $out .= ($jdw > ($doy + 3) and $doy < 3) ? $k_y - 1 : (((3 - $dny) > $jdw and $dny < 3) ? $k_y + 1 : $k_y);
        break;

      case 'y':
        $out .= substr($k_y, 2, 2);
        break;

      case 'Y':
        $out .= $k_y;
        break;

        /* Time */
      case 'H':
        $out .= $date[1];
        break;

      case 'I':
        $out .= $date[0];
        break;

      case 'l':
        $out .= ($date[0] > 9) ? $date[0] : ' ' . (int) $date[0];
        break;

      case 'M':
        $out .= $date[2];
        break;

      case 'p':
        $out .= ($date[1] < 12) ? 'پێش نیوەڕۆ' : 'پاش نیوەڕۆ';
        break;

      case 'P':
        $out .= ($date[1] < 12) ? 'پێش.ن' : 'پاش.ن';
        break;

      case 'r':
        $out .= $date[0] . ':' . $date[2] . ':' . $date[5] . ' ' . (($date[1] < 12) ? 'پێش نیوەڕۆ' : 'پاش نیوەڕۆ');
        break;

      case 'R':
        $out .= $date[1] . ':' . $date[2];
        break;

      case 'S':
        $out .= $date[5];
        break;

      case 'T':
        $out .= $date[1] . ':' . $date[2] . ':' . $date[5];
        break;

      case 'X':
        $out .= $date[0] . ':' . $date[2] . ':' . $date[5];
        break;

      case 'z':
        $out .= date('O', $ts);
        break;

      case 'Z':
        $out .= date('T', $ts);
        break;

        /* Time and Date Stamps */
      case 'c':
        $key = kdate_words(array('rh' => $date[6], 'mm' => $k_m));
        $out .= $date[1] . ':' . $date[2] . ':' . $date[5] . ' ' . date('P', $ts) . ' ' . $key['rh'] . '، ' . $k_d . ' ' . $key['mm'] . ' ' . $k_y;
        break;

      case 'D':
        $out .= substr($k_y, 2, 2) . '/' . (($k_m > 9) ? $k_m : '0' . $k_m) . '/' . (($k_d < 10) ? '0' . $k_d : $k_d);
        break;

      case 'F':
        $out .= $k_y . '-' . (($k_m > 9) ? $k_m : '0' . $k_m) . '-' . (($k_d < 10) ? '0' . $k_d : $k_d);
        break;

      case 's':
        $out .= $ts;
        break;

      case 'x':
        $out .= substr($k_y, 2, 2) . '/' . (($k_m > 9) ? $k_m : '0' . $k_m) . '/' . (($k_d < 10) ? '0' . $k_d : $k_d);
        break;

        /* Miscellaneous */
      case 'n':
        $out .= "\n";
        break;

      case 't':
        $out .= "\t";
        break;

      case '%':
        $out .= '%';
        break;

      default:
        $out .= $sub;
    }
  }
  return ($kr_num != 'en') ? kr_num($out, 'fa', '.') : $out;
}

/*	F	*/
function kmktime($h = '', $m = '', $s = '', $jm = '', $jd = '', $jy = '', $none = '', $timezone = 'Asia/Tehran') {
  if ($timezone != 'local') date_default_timezone_set($timezone);
  if ($h === '') {
    return time();
  } else {
    list($h, $m, $s, $jm, $jd, $jy) = explode('_', kr_num($h . '_' . $m . '_' . $s . '_' . $jm . '_' . $jd . '_' . $jy));
    if ($m === '') {
      return mktime($h);
    } else {
      if ($s === '') {
        return mktime($h, $m);
      } else {
        if ($jm === '') {
          return mktime($h, $m, $s);
        } else {
          $kdate = explode('_', kdate('Y_j', '', '', $timezone, 'en'));
          if ($jd === '') {
            list($gy, $gm, $gd) = kurdi_to_gregorian($kdate[0], $jm, $kdate[1]);
            return mktime($h, $m, $s, $gm);
          } else {
            if ($jy === '') {
              list($gy, $gm, $gd) = kurdi_to_gregorian($kdate[0], $jm, $jd);
              return mktime($h, $m, $s, $gm, $gd);
            } else {
              list($gy, $gm, $gd) = kurdi_to_gregorian($jy, $jm, $jd);
              return mktime($h, $m, $s, $gm, $gd, $gy);
            }
          }
        }
      }
    }
  }
}

/*	F	*/
function kgetdate($timestamp = '', $none = '', $timezone = 'Asia/Tehran', $tn = 'en') {
  $ts = ($timestamp === '') ? time() : kr_num($timestamp);
  $kdate = explode('_', kdate('F_G_i_j_l_n_s_w_Y_z', $ts, '', $timezone, $tn));
  return array(
    'seconds' => kr_num((int) kr_num($kdate[6]), $tn),
    'minutes' => kr_num((int) kr_num($kdate[2]), $tn),
    'hours' => $kdate[1],
    'mday' => $kdate[3],
    'wday' => $kdate[7],
    'mon' => $kdate[5],
    'year' => $kdate[8],
    'yday' => $kdate[9],
    'weekday' => $kdate[4],
    'month' => $kdate[0],
    0 => kr_num($ts, $tn)
  );
}

/*	F	*/
function kcheckdate($jm, $jd, $jy) {
  list($jm, $jd, $jy) = explode('_', kr_num($jm . '_' . $jd . '_' . $jy));
  $l_d = ($jm == 12 and ((($jy + 12) % 33) % 4) != 1) ? 29 : (31 - (int) ($jm / 6.5));
  return ($jm > 12 or $jd > $l_d or $jm < 1 or $jd < 1 or $jy < 1) ? false : true;
}

/*	F	*/
function kr_num($str, $mod = 'en', $mf = '٫') {
  $num_a = array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '.');
  $key_a = array('۰', '۱', '۲', '۳', '۴', '۵', '۶', '۷', '۸', '۹', $mf);
  return ($mod == 'fa') ? str_replace($num_a, $key_a, $str) : str_replace($key_a, $num_a, $str);
}

/*	F	*/
function kdate_words($array, $mod = '') {
  foreach ($array as $type => $num) {
    $num = (int) kr_num($num);
    switch ($type) {

      case 'ss':
		$number = new Number2Pit;
		$array[$type] = $number->numberToPits($num);
        break;

      case 'mm':
        $key = array('خاکەلێوە', 'بانەمەڕ', 'جۆزەردان', 'پوشپەڕ', 'گەلاوێژ', 'خەرمانان', 'ڕەزبەر', 'خەزەڵوەر', 'سەرماوەز', 'بەفرانبار', 'ڕێبەندان', 'ڕەشەمێ');
        $array[$type] = $key[$num - 1];
        break;

      case 'rr':
        $key = array(
          'یەک', 'دوو', 'سێ', 'چوار', 'پێنج', 'شەش', 'حەوت', 'هەشت', 'نۆ', 'دە', 'یازدە', 'دوازدە', 'سێزدە', 'چاردە', 'پازدە', 'شازدە', 'حەڤدە', 'هەژدە', 'نۆزدە', 'بیست', 'بیست و یەک', 'بیست و دوو', 'بیست و سێ', 'بیست و چوار', 'بیست و پێنج', 'بیست و شەش', 'بیست و حەوت', 'بیست و هەشت', 'بیست و نۆ', 'سی', 'سی و یەک'
        );
        $array[$type] = $key[$num - 1];
        break;

      case 'rh':
        $key = array('یەکشەم', 'دووشەم', 'سێشەم', 'چوارشەم', 'پێنشەم', 'هەینی', 'شەمە');
        $array[$type] = $key[$num];
        break;

      case 'sh':
        $key = array('مار', 'ئەسپ', 'مەڕ', 'مەیموون', 'مریشک', 'سەگ', 'بەراز', 'مشک', 'مانگا', 'پڵنگ', 'کەروێشک', 'نەهەنگ');
        $array[$type] = $key[($num % 12) - 1];
        break;

      case 'mb':
        $key = array('حمل', 'ثور', 'جوزا', 'سرطان', 'اسد', 'سنبله', 'میزان', 'عقرب', 'قوس', 'جدی', 'دلو', 'حوت');
        $array[$type] = $key[$num - 1];
        break;

      case 'ff':
        $key = array('بەهار', 'هاوین', 'پاییز', 'زستان');
        $array[$type] = $key[(int) ($num / 3.1)];
        break;

      case 'km':
        $key = array('خاک‍', 'بان‍', 'جۆز', 'پوو‍', 'گەل‍', 'خەر‍', 'ڕەز‍', 'خەز‍', 'سەر', 'بەف‍', 'ڕێب‍', 'ڕەش‍');
        $array[$type] = $key[$num - 1];
        break;

      case 'kh':
        $key = array('ی', 'د', 'س', 'چ', 'پ', 'ه‍', 'ش');
        $array[$type] = $key[$num];
        break;

      default:
        $array[$type] = $num;
    }
  }
  return ($mod === '') ? $array : implode($mod, $array);
}


/**  Gregorian & kurdi Date Converter Functions  **/

/*	gregorian to kurdi	*/
function gregorian_to_kurdi($gy, $gm, $gd, $mod = '') {
   list($gy, $gm, $gd) = explode('_', kr_num($gy . '_' . $gm . '_' . $gd));/* <= This line is not part of the main function: Extra */
  $g_d_m = array(0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334);
  $gy2 = ($gm > 2) ? ($gy + 1) : $gy;
  $days = 355666 + (365 * $gy) + ((int) (($gy2 + 3) / 4)) - ((int) (($gy2 + 99) / 100)) + ((int) (($gy2 + 399) / 400)) + $gd + $g_d_m[$gm - 1];
  $jy = -1595 + 1321 + (33 * ((int) ($days / 12053)));
  $days %= 12053;
  $jy += 4 * ((int) ($days / 1461));
  $days %= 1461;
  if ($days > 365) {
    $jy += (int) (($days - 1) / 365);
    $days = ($days - 1) % 365;
  }
  if ($days < 186) {
    $jm = 1 + (int) ($days / 31);
    $jd = 1 + ($days % 31);
  } else {
    $jm = 7 + (int) (($days - 186) / 30);
    $jd = 1 + (($days - 186) % 30);
  }
  return ($mod == '') ? array($jy, $jm, $jd) : $jy . $mod . $jm . $mod . $jd;
}

/*	kurdi to gregorian	*/
function kurdi_to_gregorian($jy, $jm, $jd, $mod = '') {
   list($jy, $jm, $jd) = explode('_', kr_num($jy . '_' . $jm . '_' . $jd));/* <= This line is not part of the main function: Extra */
  $jy += 1595;
  $days = -355668 + (365 * $jy) + (((int) ($jy / 33)) * 8) + ((int) ((($jy % 33) + 3) / 4)) + $jd + (($jm < 7) ? ($jm - 1) * 31 : (($jm - 7) * 30) + 186);
  $gy = 400 * ((int) ($days / 146097));
  $days %= 146097;
  if ($days > 36524) {
    $gy += 100 * ((int) (--$days / 36524));
    $days %= 36524;
    if ($days >= 365) $days++;
  }
  $gy += 4 * ((int) ($days / 1461));
  $days %= 1461;
  if ($days > 365) {
    $gy += (int) (($days - 1) / 365);
    $days = ($days - 1) % 365;
  }
  $gd = $days + 1;
  $sal_a = array(0, 31, (($gy % 4 == 0 and $gy % 100 != 0) or ($gy % 400 == 0)) ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
  for ($gm = 0; $gm < 13 and $gd > $sal_a[$gm]; $gm++) $gd -= $sal_a[$gm];
  return ($mod == '') ? array($gy, $gm, $gd) : $gy . $mod . $gm . $mod . $gd;
}


// ---------------------------------------------------------------------------------------------------


/*	F	*/
function jdate($format, $timestamp = '', $none = '', $time_zone = 'Asia/Tehran', $tr_num = 'fa') {

  $T_sec = 0;

  if ($time_zone != 'local') date_default_timezone_set(($time_zone === '') ? 'Asia/Tehran' : $time_zone);
  $ts = $T_sec + (($timestamp === '') ? time() : tr_num($timestamp));
  $date = explode('_', date('H_i_j_n_O_P_s_w_Y', $ts));
  list($j_y, $j_m, $j_d) = gregorian_to_jalali($date[8], $date[3], $date[2]);
  $doy = ($j_m < 7) ? (($j_m - 1) * 31) + $j_d - 1 : (($j_m - 7) * 30) + $j_d + 185;
  $kab = (((($j_y + 12) % 33) % 4) == 1) ? 1 : 0;
  $sl = strlen($format);
  $out = '';
  for ($i = 0; $i < $sl; $i++) {
    $sub = substr($format, $i, 1);
    if ($sub == '\\') {
      $out .= substr($format, ++$i, 1);
      continue;
    }
    switch ($sub) {

      case 'E':
      case 'R':
      case 'x':
      case 'X':
        $out .= 'http://jdf.scr.ir';
        break;

      case 'B':
      case 'e':
      case 'g':
      case 'G':
      case 'h':
      case 'I':
      case 'T':
      case 'u':
      case 'Z':
        $out .= date($sub, $ts);
        break;

      case 'a':
        $out .= ($date[0] < 12) ? 'ق.ظ' : 'ب.ظ';
        break;

      case 'A':
        $out .= ($date[0] < 12) ? 'قبل از ظهر' : 'بعد از ظهر';
        break;

      case 'b':
        $out .= (int) ($j_m / 3.1) + 1;
        break;

      case 'c':
        $out .= $j_y . '/' . $j_m . '/' . $j_d . ' ،' . $date[0] . ':' . $date[1] . ':' . $date[6] . ' ' . $date[5];
        break;

      case 'C':
        $out .= (int) (($j_y + 99) / 100);
        break;

      case 'd':
        $out .= ($j_d < 10) ? '0' . $j_d : $j_d;
        break;

      case 'D':
        $out .= jdate_words(array('kh' => $date[7]), ' ');
        break;

      case 'f':
        $out .= jdate_words(array('ff' => $j_m), ' ');
        break;

      case 'F':
        $out .= jdate_words(array('mm' => $j_m), ' ');
        break;

      case 'H':
        $out .= $date[0];
        break;

      case 'i':
        $out .= $date[1];
        break;

      case 'j':
        $out .= $j_d;
        break;

      case 'J':
        $out .= jdate_words(array('rr' => $j_d), ' ');
        break;

      case 'k';
        $out .= tr_num(100 - (int) ($doy / ($kab + 365.24) * 1000) / 10, $tr_num);
        break;

      case 'K':
        $out .= tr_num((int) ($doy / ($kab + 365.24) * 1000) / 10, $tr_num);
        break;

      case 'l':
        $out .= jdate_words(array('rh' => $date[7]), ' ');
        break;

      case 'L':
        $out .= $kab;
        break;

      case 'm':
        $out .= ($j_m > 9) ? $j_m : '0' . $j_m;
        break;

      case 'M':
        $out .= jdate_words(array('km' => $j_m), ' ');
        break;

      case 'n':
        $out .= $j_m;
        break;

      case 'N':
        $out .= $date[7] + 1;
        break;

      case 'o':
        $jdw = ($date[7] == 6) ? 0 : $date[7] + 1;
        $dny = 364 + $kab - $doy;
        $out .= ($jdw > ($doy + 3) and $doy < 3) ? $j_y - 1 : (((3 - $dny) > $jdw and $dny < 3) ? $j_y + 1 : $j_y);
        break;

      case 'O':
        $out .= $date[4];
        break;

      case 'p':
        $out .= jdate_words(array('mb' => $j_m), ' ');
        break;

      case 'P':
        $out .= $date[5];
        break;

      case 'q':
        $out .= jdate_words(array('sh' => $j_y), ' ');
        break;

      case 'Q':
        $out .= $kab + 364 - $doy;
        break;

      case 'r':
        $key = jdate_words(array('rh' => $date[7], 'mm' => $j_m));
        $out .= $date[0] . ':' . $date[1] . ':' . $date[6] . ' ' . $date[4] . ' ' . $key['rh'] . '، ' . $j_d . ' ' . $key['mm'] . ' ' . $j_y;
        break;

      case 's':
        $out .= $date[6];
        break;

      case 'S':
        $out .= 'ام';
        break;

      case 't':
        $out .= ($j_m != 12) ? (31 - (int) ($j_m / 6.5)) : ($kab + 29);
        break;

      case 'U':
        $out .= $ts;
        break;

      case 'v':
        $out .= jdate_words(array('ss' => ($j_y % 100)), ' ');
        break;

      case 'V':
        $out .= jdate_words(array('ss' => $j_y), ' ');
        break;

      case 'w':
        $out .= ($date[7] == 6) ? 0 : $date[7] + 1;
        break;

      case 'W':
        $avs = (($date[7] == 6) ? 0 : $date[7] + 1) - ($doy % 7);
        if ($avs < 0) $avs += 7;
        $num = (int) (($doy + $avs) / 7);
        if ($avs < 4) {
          $num++;
        } elseif ($num < 1) {
          $num = ($avs == 4 or $avs == ((((($j_y % 33) % 4) - 2) == ((int) (($j_y % 33) * 0.05))) ? 5 : 4)) ? 53 : 52;
        }
        $aks = $avs + $kab;
        if ($aks == 7) $aks = 0;
        $out .= (($kab + 363 - $doy) < $aks and $aks < 3) ? '01' : (($num < 10) ? '0' . $num : $num);
        break;

      case 'y':
        $out .= substr($j_y, 2, 2);
        break;

      case 'Y':
        $out .= $j_y;
        break;

      case 'z':
        $out .= $doy;
        break;

      default:
        $out .= $sub;
    }
  }
  return ($tr_num != 'en') ? tr_num($out, 'fa', '.') : $out;
}

/*	F	*/
function jstrftime($format, $timestamp = '', $none = '', $time_zone = 'Asia/Tehran', $tr_num = 'fa') {

  $T_sec = 0;

  if ($time_zone != 'local') date_default_timezone_set(($time_zone === '') ? 'Asia/Tehran' : $time_zone);
  $ts = $T_sec + (($timestamp === '') ? time() : tr_num($timestamp));
  $date = explode('_', date('h_H_i_j_n_s_w_Y', $ts));
  list($j_y, $j_m, $j_d) = gregorian_to_jalali($date[7], $date[4], $date[3]);
  $doy = ($j_m < 7) ? (($j_m - 1) * 31) + $j_d - 1 : (($j_m - 7) * 30) + $j_d + 185;
  $kab = (((($j_y + 12) % 33) % 4) == 1) ? 1 : 0;
  $sl = strlen($format);
  $out = '';
  for ($i = 0; $i < $sl; $i++) {
    $sub = substr($format, $i, 1);
    if ($sub == '%') {
      $sub = substr($format, ++$i, 1);
    } else {
      $out .= $sub;
      continue;
    }
    switch ($sub) {

        /* Day */
      case 'a':
        $out .= jdate_words(array('kh' => $date[6]), ' ');
        break;

      case 'A':
        $out .= jdate_words(array('rh' => $date[6]), ' ');
        break;

      case 'd':
        $out .= ($j_d < 10) ? '0' . $j_d : $j_d;
        break;

      case 'e':
        $out .= ($j_d < 10) ? ' ' . $j_d : $j_d;
        break;

      case 'j':
        $out .= str_pad($doy + 1, 3, 0, STR_PAD_LEFT);
        break;

      case 'u':
        $out .= $date[6] + 1;
        break;

      case 'w':
        $out .= ($date[6] == 6) ? 0 : $date[6] + 1;
        break;

        /* Week */
      case 'U':
        $avs = (($date[6] < 5) ? $date[6] + 2 : $date[6] - 5) - ($doy % 7);
        if ($avs < 0) $avs += 7;
        $num = (int) (($doy + $avs) / 7) + 1;
        if ($avs > 3 or $avs == 1) $num--;
        $out .= ($num < 10) ? '0' . $num : $num;
        break;

      case 'V':
        $avs = (($date[6] == 6) ? 0 : $date[6] + 1) - ($doy % 7);
        if ($avs < 0) $avs += 7;
        $num = (int) (($doy + $avs) / 7);
        if ($avs < 4) {
          $num++;
        } elseif ($num < 1) {
          $num = ($avs == 4 or $avs == ((((($j_y % 33) % 4) - 2) == ((int) (($j_y % 33) * 0.05))) ? 5 : 4)) ? 53 : 52;
        }
        $aks = $avs + $kab;
        if ($aks == 7) $aks = 0;
        $out .= (($kab + 363 - $doy) < $aks and $aks < 3) ? '01' : (($num < 10) ? '0' . $num : $num);
        break;

      case 'W':
        $avs = (($date[6] == 6) ? 0 : $date[6] + 1) - ($doy % 7);
        if ($avs < 0) $avs += 7;
        $num = (int) (($doy + $avs) / 7) + 1;
        if ($avs > 3) $num--;
        $out .= ($num < 10) ? '0' . $num : $num;
        break;

        /* Month */
      case 'b':
      case 'h':
        $out .= jdate_words(array('km' => $j_m), ' ');
        break;

      case 'B':
        $out .= jdate_words(array('mm' => $j_m), ' ');
        break;

      case 'm':
        $out .= ($j_m > 9) ? $j_m : '0' . $j_m;
        break;

        /* Year */
      case 'C':
        $tmp = (int) ($j_y / 100);
        $out .= ($tmp > 9) ? $tmp : '0' . $tmp;
        break;

      case 'g':
        $jdw = ($date[6] == 6) ? 0 : $date[6] + 1;
        $dny = 364 + $kab - $doy;
        $out .= substr(($jdw > ($doy + 3) and $doy < 3) ? $j_y - 1 : (((3 - $dny) > $jdw and $dny < 3) ? $j_y + 1 : $j_y), 2, 2);
        break;

      case 'G':
        $jdw = ($date[6] == 6) ? 0 : $date[6] + 1;
        $dny = 364 + $kab - $doy;
        $out .= ($jdw > ($doy + 3) and $doy < 3) ? $j_y - 1 : (((3 - $dny) > $jdw and $dny < 3) ? $j_y + 1 : $j_y);
        break;

      case 'y':
        $out .= substr($j_y, 2, 2);
        break;

      case 'Y':
        $out .= $j_y;
        break;

        /* Time */
      case 'H':
        $out .= $date[1];
        break;

      case 'I':
        $out .= $date[0];
        break;

      case 'l':
        $out .= ($date[0] > 9) ? $date[0] : ' ' . (int) $date[0];
        break;

      case 'M':
        $out .= $date[2];
        break;

      case 'p':
        $out .= ($date[1] < 12) ? 'قبل از ظهر' : 'بعد از ظهر';
        break;

      case 'P':
        $out .= ($date[1] < 12) ? 'ق.ظ' : 'ب.ظ';
        break;

      case 'r':
        $out .= $date[0] . ':' . $date[2] . ':' . $date[5] . ' ' . (($date[1] < 12) ? 'قبل از ظهر' : 'بعد از ظهر');
        break;

      case 'R':
        $out .= $date[1] . ':' . $date[2];
        break;

      case 'S':
        $out .= $date[5];
        break;

      case 'T':
        $out .= $date[1] . ':' . $date[2] . ':' . $date[5];
        break;

      case 'X':
        $out .= $date[0] . ':' . $date[2] . ':' . $date[5];
        break;

      case 'z':
        $out .= date('O', $ts);
        break;

      case 'Z':
        $out .= date('T', $ts);
        break;

        /* Time and Date Stamps */
      case 'c':
        $key = jdate_words(array('rh' => $date[6], 'mm' => $j_m));
        $out .= $date[1] . ':' . $date[2] . ':' . $date[5] . ' ' . date('P', $ts) . ' ' . $key['rh'] . '، ' . $j_d . ' ' . $key['mm'] . ' ' . $j_y;
        break;

      case 'D':
        $out .= substr($j_y, 2, 2) . '/' . (($j_m > 9) ? $j_m : '0' . $j_m) . '/' . (($j_d < 10) ? '0' . $j_d : $j_d);
        break;

      case 'F':
        $out .= $j_y . '-' . (($j_m > 9) ? $j_m : '0' . $j_m) . '-' . (($j_d < 10) ? '0' . $j_d : $j_d);
        break;

      case 's':
        $out .= $ts;
        break;

      case 'x':
        $out .= substr($j_y, 2, 2) . '/' . (($j_m > 9) ? $j_m : '0' . $j_m) . '/' . (($j_d < 10) ? '0' . $j_d : $j_d);
        break;

        /* Miscellaneous */
      case 'n':
        $out .= "\n";
        break;

      case 't':
        $out .= "\t";
        break;

      case '%':
        $out .= '%';
        break;

      default:
        $out .= $sub;
    }
  }
  return ($tr_num != 'en') ? tr_num($out, 'fa', '.') : $out;
}

/*	F	*/
function jmktime($h = '', $m = '', $s = '', $jm = '', $jd = '', $jy = '', $none = '', $timezone = 'Asia/Tehran') {
  if ($timezone != 'local') date_default_timezone_set($timezone);
  if ($h === '') {
    return time();
  } else {
    list($h, $m, $s, $jm, $jd, $jy) = explode('_', tr_num($h . '_' . $m . '_' . $s . '_' . $jm . '_' . $jd . '_' . $jy));
    if ($m === '') {
      return mktime($h);
    } else {
      if ($s === '') {
        return mktime($h, $m);
      } else {
        if ($jm === '') {
          return mktime($h, $m, $s);
        } else {
          $jdate = explode('_', jdate('Y_j', '', '', $timezone, 'en'));
          if ($jd === '') {
            list($gy, $gm, $gd) = jalali_to_gregorian($jdate[0], $jm, $jdate[1]);
            return mktime($h, $m, $s, $gm);
          } else {
            if ($jy === '') {
              list($gy, $gm, $gd) = jalali_to_gregorian($jdate[0], $jm, $jd);
              return mktime($h, $m, $s, $gm, $gd);
            } else {
              list($gy, $gm, $gd) = jalali_to_gregorian($jy, $jm, $jd);
              return mktime($h, $m, $s, $gm, $gd, $gy);
            }
          }
        }
      }
    }
  }
}

/*	F	*/
function jgetdate($timestamp = '', $none = '', $timezone = 'Asia/Tehran', $tn = 'en') {
  $ts = ($timestamp === '') ? time() : tr_num($timestamp);
  $jdate = explode('_', jdate('F_G_i_j_l_n_s_w_Y_z', $ts, '', $timezone, $tn));
  return array(
    'seconds' => tr_num((int) tr_num($jdate[6]), $tn),
    'minutes' => tr_num((int) tr_num($jdate[2]), $tn),
    'hours' => $jdate[1],
    'mday' => $jdate[3],
    'wday' => $jdate[7],
    'mon' => $jdate[5],
    'year' => $jdate[8],
    'yday' => $jdate[9],
    'weekday' => $jdate[4],
    'month' => $jdate[0],
    0 => tr_num($ts, $tn)
  );
}

/*	F	*/
function jcheckdate($jm, $jd, $jy) {
  list($jm, $jd, $jy) = explode('_', tr_num($jm . '_' . $jd . '_' . $jy));
  $l_d = ($jm == 12 and ((($jy + 12) % 33) % 4) != 1) ? 29 : (31 - (int) ($jm / 6.5));
  return ($jm > 12 or $jd > $l_d or $jm < 1 or $jd < 1 or $jy < 1) ? false : true;
}

/*	F	*/
function tr_num($str, $mod = 'en', $mf = '٫') {
  $num_a = array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '.');
  $key_a = array('۰', '۱', '۲', '۳', '۴', '۵', '۶', '۷', '۸', '۹', $mf);
  return ($mod == 'fa') ? str_replace($num_a, $key_a, $str) : str_replace($key_a, $num_a, $str);
}

/*	F	*/
function jdate_words($array, $mod = '') {
  foreach ($array as $type => $num) {
    $num = (int) tr_num($num);
    switch ($type) {

      case 'ss':
        $sl = strlen($num);
        $xy3 = substr($num, 2 - $sl, 1);
        $h3 = $h34 = $h4 = '';
        if ($xy3 == 1) {
          $p34 = '';
          $k34 = array('ده', 'یازده', 'دوازده', 'سیزده', 'چهارده', 'پانزده', 'شانزده', 'هفده', 'هجده', 'نوزده');
          $h34 = $k34[substr($num, 2 - $sl, 2) - 10];
        } else {
          $xy4 = substr($num, 3 - $sl, 1);
          $p34 = ($xy3 == 0 or $xy4 == 0) ? '' : ' و ';
          $k3 = array('', '', 'بیست', 'سی', 'چهل', 'پنجاه', 'شصت', 'هفتاد', 'هشتاد', 'نود');
          $h3 = $k3[$xy3];
          $k4 = array('', 'یک', 'دو', 'سه', 'چهار', 'پنج', 'شش', 'هفت', 'هشت', 'نه');
          $h4 = $k4[$xy4];
        }
        $array[$type] = (($num > 99) ? str_replace(
          array('12', '13', '14', '19', '20'),
          array('هزار و دویست', 'هزار و سیصد', 'هزار و چهارصد', 'هزار و نهصد', 'دوهزار'),
          substr($num, 0, 2)
        ) . ((substr($num, 2, 2) == '00') ? '' : ' و ') : '') . $h3 . $p34 . $h34 . $h4;
        break;

      case 'mm':
        $key = array('فروردین', 'اردیبهشت', 'خرداد', 'تیر', 'مرداد', 'شهریور', 'مهر', 'آبان', 'آذر', 'دی', 'بهمن', 'اسفند');
        $array[$type] = $key[$num - 1];
        break;

      case 'rr':
        $key = array(
          'یک', 'دو', 'سه', 'چهار', 'پنج', 'شش', 'هفت', 'هشت', 'نه', 'ده', 'یازده', 'دوازده', 'سیزده', 'چهارده', 'پانزده', 'شانزده', 'هفده', 'هجده', 'نوزده', 'بیست', 'بیست و یک', 'بیست و دو', 'بیست و سه', 'بیست و چهار', 'بیست و پنج', 'بیست و شش', 'بیست و هفت', 'بیست و هشت', 'بیست و نه', 'سی', 'سی و یک'
        );
        $array[$type] = $key[$num - 1];
        break;

      case 'rh':
        $key = array('یکشنبه', 'دوشنبه', 'سه شنبه', 'چهارشنبه', 'پنجشنبه', 'جمعه', 'شنبه');
        $array[$type] = $key[$num];
        break;

      case 'sh':
        $key = array('مار', 'اسب', 'گوسفند', 'میمون', 'مرغ', 'سگ', 'خوک', 'موش', 'گاو', 'پلنگ', 'خرگوش', 'نهنگ');
        $array[$type] = $key[$num % 12];
        break;

      case 'mb':
        $key = array('حمل', 'ثور', 'جوزا', 'سرطان', 'اسد', 'سنبله', 'میزان', 'عقرب', 'قوس', 'جدی', 'دلو', 'حوت');
        $array[$type] = $key[$num - 1];
        break;

      case 'ff':
        $key = array('بهار', 'تابستان', 'پاییز', 'زمستان');
        $array[$type] = $key[(int) ($num / 3.1)];
        break;

      case 'km':
        $key = array('فر', 'ار', 'خر', 'تی‍', 'مر', 'شه‍', 'مه‍', 'آب‍', 'آذ', 'دی', 'به‍', 'اس‍');
        $array[$type] = $key[$num - 1];
        break;

      case 'kh':
        $key = array('ی', 'د', 'س', 'چ', 'پ', 'ج', 'ش');
        $array[$type] = $key[$num];
        break;

      default:
        $array[$type] = $num;
    }
  }
  return ($mod === '') ? $array : implode($mod, $array);
}



/*	F	*/
function gregorian_to_jalali($gy, $gm, $gd, $mod = '') {
   list($gy, $gm, $gd) = explode('_', tr_num($gy . '_' . $gm . '_' . $gd));
  $g_d_m = array(0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334);
  $gy2 = ($gm > 2) ? ($gy + 1) : $gy;
  $days = 355666 + (365 * $gy) + ((int) (($gy2 + 3) / 4)) - ((int) (($gy2 + 99) / 100)) + ((int) (($gy2 + 399) / 400)) + $gd + $g_d_m[$gm - 1];
  $jy = -1595 + (33 * ((int) ($days / 12053)));
  $days %= 12053;
  $jy += 4 * ((int) ($days / 1461));
  $days %= 1461;
  if ($days > 365) {
    $jy += (int) (($days - 1) / 365);
    $days = ($days - 1) % 365;
  }
  if ($days < 186) {
    $jm = 1 + (int) ($days / 31);
    $jd = 1 + ($days % 31);
  } else {
    $jm = 7 + (int) (($days - 186) / 30);
    $jd = 1 + (($days - 186) % 30);
  }
  return ($mod == '') ? array($jy, $jm, $jd) : $jy . $mod . $jm . $mod . $jd;
}

/*	F	*/
function jalali_to_gregorian($jy, $jm, $jd, $mod = '') {
   list($jy, $jm, $jd) = explode('_', tr_num($jy . '_' . $jm . '_' . $jd));/* <= Extra :اين سطر ، جزء تابع اصلي نيست */
  $jy += 1595;
  $days = -355668 + (365 * $jy) + (((int) ($jy / 33)) * 8) + ((int) ((($jy % 33) + 3) / 4)) + $jd + (($jm < 7) ? ($jm - 1) * 31 : (($jm - 7) * 30) + 186);
  $gy = 400 * ((int) ($days / 146097));
  $days %= 146097;
  if ($days > 36524) {
    $gy += 100 * ((int) (--$days / 36524));
    $days %= 36524;
    if ($days >= 365) $days++;
  }
  $gy += 4 * ((int) ($days / 1461));
  $days %= 1461;
  if ($days > 365) {
    $gy += (int) (($days - 1) / 365);
    $days = ($days - 1) % 365;
  }
  $gd = $days + 1;
  $sal_a = array(0, 31, (($gy % 4 == 0 and $gy % 100 != 0) or ($gy % 400 == 0)) ? 29 : 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
  for ($gm = 0; $gm < 13 and $gd > $sal_a[$gm]; $gm++) $gd -= $sal_a[$gm];
  return ($mod == '') ? array($gy, $gm, $gd) : $gy . $mod . $gm . $mod . $gd;
}
class Number2Pit{protected $digit0=array(0=>'سفر',1=>'یەک',2=>'دوو',3=>'سێ',4=>'چوار',5=>'پێنج',6=>'شەش',7=>'حەوت',8=>'هەشت',9=>'نۆ',);protected $digit1=array(1=>'یازدە',2=>'دوازدە',3=>'سێزدە',4=>'چواردە',5=>'پازدە',6=>'شازدە',7=>'حەڤدە',8=>'هەژدە',9=>'نۆزدە',);protected $digit2=array(1=>'دە',2=>'بیست',3=>'سی',4=>'چل',5=>'پەنجا',6=>'شێست',7=>'حەفتا',8=>'هەشتا',9=>'نەوەد');protected $digit3=array(1=>'سەد',2=>'دووسەد',3=>'سێسەد',4=>'چوارسەد',5=>'پێنسەد',6=>'شەشسەد',7=>'حەوسەد',8=>'هەشسەد',9=>'نۆسەد',);protected $steps=array(1=>'هەزار',2=>'ملیۆن',3=>'بیلیۆن',4=>'تریلیۆن',5=>'کادریلیۆن',6=>'کۆینتریلیۆن',7=>'سکستریلیۆن',8=>'سەپتریلیۆن',9=>'ئۆکتریلیۆن',10=>'نۆنیلیۆن',11=>'دەسیلیۆن',);protected $t=array('and'=>'و',);function number_format($number,$decimal_precision=0,$decimals_separator='.',$thousands_separator=','){$number=explode('.',str_replace(' ','',$number));$number[0]=str_split(strrev($number[0]),3);$total_segments=count($number[0]);for($i=0;$i<$total_segments;$i++){$number[0][$i]=strrev($number[0][$i]);}$number[0]=implode($thousands_separator,array_reverse($number[0]));if(!empty($number[1])){$number[1]=round($number[1],$decimal_precision);}return implode($decimals_separator,$number);}protected function groupToWords($group){$d3=floor($group/100);$d2=floor(($group-$d3*100)/10);$d1=$group-$d3*100-$d2*10;$group_array=array();if($d3!=0){$group_array[]=$this->digit3[$d3];}if($d2==1&&$d1!=0){$group_array[]=$this->digit1[$d1];}else if($d2!=0&&$d1==0){$group_array[]=$this->digit2[$d2];}else if($d2==0&&$d1==0){}else if($d2==0&&$d1!=0){$group_array[]=$this->digit0[$d1];}else{$group_array[]=$this->digit2[$d2];$group_array[]=$this->digit0[$d1];}if(!count($group_array)){return FALSE;}return $group_array;}public function numberToPits($number){$formated=$this->number_format($number,0,'.',',');$groups=explode(',',$formated);$steps=count($groups);$parts=array();foreach($groups as $step=>$group){$group_words=self::groupToWords($group);if($group_words){$part=implode(' '.$this->t['and'].' ',$group_words);if(isset($this->steps[$steps-$step-1])){$part.=' '.$this->steps[$steps-$step-1];}$parts[]=$part;}}return implode(' '.$this->t['and'].' ',$parts);}}
?>